#include "OfferwallView.h"
#include "../../../../pokktsdk/PokktManager.h"

using namespace cocos2d;
using namespace ui;

bool OfferwallView::init()
{
    if (!BaseView::init())
        return false;
    
    _coinsEarned = 0;
    
    logo->setPosition(logo->getPosition().x, logo->getPosition().y + 100);
    
    int yPos = logo->getPosition().y - logo->getBoundingBox().size.height / 2;
    
    
    auto lblPointEarned = BaseView::createLabel("Points Earned: ", 35, Color3B::BLACK, Vec2(visibleSize.width / 2 - 30, yPos));
    lblPointEarned->setTextHorizontalAlignment(TextHAlignment::RIGHT);
    this->addChild(lblPointEarned);
    
    _txtPointsEarned = BaseView::createLabel("0", 35, Color3B::BLACK,
                                             Vec2(lblPointEarned->getBoundingBox().size.width / 2 + lblPointEarned->getPosition().x + 20, yPos));
    this->addChild(_txtPointsEarned);
    
    yPos -= 100;
    
    _btnGetOfferwallAny = createButton("Get Offerwall (Any Asset Value)", CC_CALLBACK_1(OfferwallView::handleGetOfferwallAny, this));
    _btnGetOfferwallAny->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnGetOfferwallAny);
    
    yPos -= 100;
    
    _txiEnterMaxPoints = TextField::create();
    _txiEnterMaxPoints->setPlaceHolder("Enter Max Points");
    _txiEnterMaxPoints->setPlaceHolderColor(Color3B::GRAY);
    _txiEnterMaxPoints->ignoreContentAdaptWithSize(false);
    _txiEnterMaxPoints->setFontSize(30);
    _txiEnterMaxPoints->setPosition(Vec2(visibleSize.width / 2, yPos));
    _txiEnterMaxPoints->setSize(Size(500, 50));
    _txiEnterMaxPoints->addEventListener(CC_CALLBACK_1(OfferwallView::handleTextInput, this));
    _txiEnterMaxPoints->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txiEnterMaxPoints);
    
    yPos -= 80;
    
    _btnGetOfferwallFix = createButton("Get Offerwall (Fix asset value)", CC_CALLBACK_1(OfferwallView::handleGetOfferwallFix, this), "button_purple.png");
    _btnGetOfferwallFix->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnGetOfferwallFix);
    
    yPos -= 100;
    
    _btnIncompleteCallback = createButton("Incomplete callback", CC_CALLBACK_1(OfferwallView::handleIncompleteCallback, this));
    _btnIncompleteCallback->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnIncompleteCallback);
    
    yPos -= 100;
    
    
    auto lblCheckBox = createLabel("Close on Success: ", 30, Color3B::BLACK, Vec2(visibleSize.width / 2 - 30, yPos));
    lblCheckBox->setTextHorizontalAlignment(TextHAlignment::RIGHT);
    this->addChild(lblCheckBox);
    
    _chkCloseOnSuccess = CheckBox::create();
    _chkCloseOnSuccess->setTouchEnabled(true);
    _chkCloseOnSuccess->setScale(0.8f);
    _chkCloseOnSuccess->loadTextures("check_box_normal.png",
                                     "check_box_normal_press.png",
                                     "check_box_active.png",
                                     "check_box_normal_disable.png",
                                     "check_box_active_disable.png");
    _chkCloseOnSuccess->setPosition(Vec2(lblCheckBox->getPosition().x + lblCheckBox->getBoundingBox().size.width / 2 + 30, yPos));
    _chkCloseOnSuccess->addEventListener(CC_CALLBACK_1(OfferwallView::handleCheckBox, this));
    this->addChild(_chkCloseOnSuccess);
    
    // listen for pokkt events
    _eventDispatcher->addCustomEventListener(PokktManager::CoinResponseEvent, CC_CALLBACK_1(OfferwallView::handleCoinResponse, this));
    _eventDispatcher->addCustomEventListener(PokktManager::CoinResponseWithTransIdEvent, CC_CALLBACK_1(OfferwallView::handleCoinResponseWithTrId, this));
    _eventDispatcher->addCustomEventListener(PokktManager::CoinResponseFailedEvent, CC_CALLBACK_1(OfferwallView::handleCoinResponseFailed, this));
    _eventDispatcher->addCustomEventListener(PokktManager::CampaignAvailabilityEvent, CC_CALLBACK_1(OfferwallView::handleCampaignAvailibity, this));
    _eventDispatcher->addCustomEventListener(PokktManager::OfferwallClosedEvent, CC_CALLBACK_1(OfferwallView::handleOfferwallclosed, this));
    
    PokktManager::getInstance()->setThirdPartyUserId("123456");
    PokktManager::getInstance()->checkOfferwallCampaign();
    PokktManager::getInstance()->getPendingCoins();
    
    return true;
}

OfferwallView::~OfferwallView()
{
    _eventDispatcher->removeCustomEventListeners(PokktManager::CoinResponseEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::CoinResponseWithTransIdEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::CoinResponseFailedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::CampaignAvailabilityEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::OfferwallClosedEvent);
}

void OfferwallView::handleTextInput(cocos2d::Ref *pSender)
{
    CCLOG("text input");
}

void OfferwallView::handleCheckBox(cocos2d::Ref *pSender)
{
    CCLOG("checkbox event");
}

void OfferwallView::handleBackPress(cocos2d::Ref *pSender)
{
    changeView(MainView);
}

void OfferwallView::handleEnterForeground()
{
    PokktManager::getInstance()->getPendingCoins();
}

void OfferwallView::handleGetOfferwallAny(cocos2d::Ref *pSender)
{
    PokktManager::getInstance()->getFreeCoins();
}

void OfferwallView::handleGetOfferwallFix(cocos2d::Ref *pSender)
{
    auto fixedAssetValue = _txiEnterMaxPoints->getString();
    PokktManager::getInstance()->getFreeCoins(fixedAssetValue);
}

void OfferwallView::handleIncompleteCallback(cocos2d::Ref *pSender)
{
    PokktManager::getInstance()->getPendingCoins();
    
    _txtPointsEarned = 0;
}

void OfferwallView::handleCoinResponse(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto param = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] Coins earned from offerwall: %s", param.c_str());
    
    int coins = 0;
    std::stringstream(param) >> coins;
    _coinsEarned += coins;
    
    char buf[100];
    sprintf(buf, "%d", _coinsEarned);
    _txtPointsEarned->setString(buf);
    
    // if auto-close is enables, close this screen
    if (_chkCloseOnSuccess->isSelected())
        handleBackPress(NULL);
}

void OfferwallView::handleCoinResponseWithTrId(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto param = *static_cast<std::string*>(event->getUserData());
    
    // extract comma separated values
    std::stringstream stream(param);
    std::string value;
    std::vector<std::string> values;
    while (getline(stream, value, ','))
    {
        values.push_back(value);
        if (stream.peek() == ',')
            stream.ignore();
    }
    
    if (values.size() < 2)
        return; // the values received are not proper
    
    auto points = values[0];
    auto transId = values[1];
    
    if (points == "-1")
    {
        // no points earned
    }
    else
    {
        // points earned equal to coins with transaction id
    }

    
    if (values.size() >= 2)
        CCLOG("[POKKT-CPP] Coins earned with trId: %s, coins: %s.", values[1].c_str(), values[0].c_str());
    else
        CCLOG("[POKKT-CPP] Error receiving coins with transaction id!");
    
    _txtPointsEarned->setString(param);
    
    // if auto-close is enables, close this screen
    if (_chkCloseOnSuccess->isSelected())
        handleBackPress(NULL);
}

void OfferwallView::handleCoinResponseFailed(cocos2d::Ref* pSender)
{
    // pending coins request fails
    CCLOG("[POKKT-CPP] Failed to get Pending coins!");
}

void OfferwallView::handleCampaignAvailibity(cocos2d::Ref *pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto param = *static_cast<std::string*>(event->getUserData());
    bool available = param == "true";
    
    CCLOG("[POKKT-CPP] Offerwall Campaign availability: %s", (available ? "available" : "no campaigns"));
}

void OfferwallView::handleOfferwallclosed(cocos2d::Ref *pSender)
{
    CCLOG("[POKKT-CPP] Offerwall Closed!");
}
