#include "VideoView.h"
#include "../../../../pokktsdk/PokktManager.h"

using namespace cocos2d;
using namespace ui;

bool VideoView::init()
{
    if (!BaseView::init())
        return false;
    
    _coinsEarned = 0;
    
    logo->setPosition(logo->getPosition().x, logo->getPosition().y + 100);
    
    int yPos = logo->getPosition().y - logo->getBoundingBox().size.height / 2;
    
    auto lblPointEarned = BaseView::createLabel("Points Earned: ", 35, Color3B::BLACK, Vec2(visibleSize.width / 2 - 30, yPos));
    lblPointEarned->setTextHorizontalAlignment(TextHAlignment::RIGHT);
    this->addChild(lblPointEarned);
    
    _txtPointsEarned = BaseView::createLabel("0", 35, Color3B::BLACK,
                                             Vec2(lblPointEarned->getBoundingBox().size.width / 2 + lblPointEarned->getPosition().x + 20, yPos));
    this->addChild(_txtPointsEarned);
    
    yPos -= 100;
    
    _txtCacheMessage = createLabel("Getting Video Campaign, please wait...", 25, Color3B::WHITE, Vec2(visibleSize.width / 2, yPos));
    _txtCacheMessage->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txtCacheMessage);
    
    yPos -= 120;
    
    _btnStartCaching = createButton("Start Caching", CC_CALLBACK_1(VideoView::handleStartCaching, this));
    _btnStartCaching->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnStartCaching->setVisible(!PokktManager::getInstance()->getAutoCaching());
    this->addChild(_btnStartCaching);
    
    yPos -= 100;
    
    _btnPlayVideo = createButton("Play Video to Earn Coins", CC_CALLBACK_1(VideoView::handlePlayVideo, this));
    _btnPlayVideo->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnPlayVideo->setVisible(false);
    this->addChild(_btnPlayVideo);
    
    yPos -= 100;
    
    _btnPlayVideoNonIncent = createButton("Play Non Incentive Video", CC_CALLBACK_1(VideoView::handlePlayVideoNonIncent, this), "button_purple.png");
    _btnPlayVideoNonIncent->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnPlayVideoNonIncent->setVisible(false);
    this->addChild(_btnPlayVideoNonIncent);
    
    yPos -= 100;
    
    _btnPlayVideoWithIntent = createButton("Play Video With Intent", CC_CALLBACK_1(VideoView::handlePlayVideoWithIntent, this));
    _btnPlayVideoWithIntent->setPosition(Vec2(visibleSize.width / 2, yPos));
    //_btnPlayVideoWithIntent->setVisible(false);
    this->addChild(_btnPlayVideoWithIntent);
    
    // listen for pokkt events
    _eventDispatcher->addCustomEventListener(PokktManager::VideoClosedEvent, CC_CALLBACK_1(VideoView::handleVideoClosed, this));
    _eventDispatcher->addCustomEventListener(PokktManager::VideoDisplayedEvent, CC_CALLBACK_1(VideoView::handleVideoDisplayed, this));
    _eventDispatcher->addCustomEventListener(PokktManager::VideoSkippedEvent, CC_CALLBACK_1(VideoView::handleVideoSkipped, this));
    _eventDispatcher->addCustomEventListener(PokktManager::VideoCompletedEvent, CC_CALLBACK_1(VideoView::handleVideoCompleted, this));
    _eventDispatcher->addCustomEventListener(PokktManager::VideoGratifiedEvent, CC_CALLBACK_1(VideoView::handleVideoGratified, this));
    
    _eventDispatcher->addCustomEventListener(PokktManager::DownloadCompletedEvent, CC_CALLBACK_1(VideoView::handleDownloadCompleted, this));
    _eventDispatcher->addCustomEventListener(PokktManager::DownloadFailedEvent, CC_CALLBACK_1(VideoView::handleDownloadFailed, this));
    
    PokktManager::getInstance()->setThirdPartyUserId("123456");
    PokktManager::getInstance()->setDefaultSkipTime("10");
    
    return true;
}

VideoView::~VideoView()
{
    _eventDispatcher->removeCustomEventListeners(PokktManager::VideoClosedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::VideoDisplayedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::VideoSkippedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::VideoCompletedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::VideoGratifiedEvent);
    
    _eventDispatcher->removeCustomEventListeners(PokktManager::DownloadCompletedEvent);
    _eventDispatcher->removeCustomEventListeners(PokktManager::DownloadFailedEvent);
}

void VideoView::handleBackPress(cocos2d::Ref *pSender)
{
    changeView(MainView);
}

void VideoView::handleStartCaching(cocos2d::Ref *pSender)
{
    PokktManager::getInstance()->startVideoCaching();
}

void VideoView::handlePlayVideo(cocos2d::Ref *pSender)
{
    PokktManager::getInstance()->getVideo();
}

void VideoView::handlePlayVideoNonIncent(cocos2d::Ref *pSender)
{
    PokktManager::getInstance()->getVideoNonIncent();
}

void VideoView::handlePlayVideoWithIntent(cocos2d::Ref *pSender)
{
    if (PokktManager::getInstance()->isVideoAvailable())
        PokktManager::getInstance()->getVideoWithIntent();
    else
        changeView(AppView::IntentView);
}

void VideoView::handleVideoClosed(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto message = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message);
}

void VideoView::handleVideoDisplayed(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto message = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message);
}

void VideoView::handleVideoSkipped(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto message = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message);
}

void VideoView::handleVideoCompleted(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto message = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message);
}

void VideoView::handleVideoGratified(cocos2d::Ref* pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto points = *static_cast<std::string*>(event->getUserData());
    
    CCLOG("[POKKT-CPP] Coins earned from video: %s", points.c_str());
    
    int coins = 0;
    std::stringstream(points) >> coins;
    _coinsEarned += coins;
    
    char buf[100];
    sprintf(buf, "%d", _coinsEarned);
    _txtPointsEarned->setString(buf);
}

void VideoView::handleDownloadCompleted(cocos2d::Ref *pSender)
{
    auto event = static_cast<EventCustom*>(pSender);
    auto coinsToEarn = *static_cast<std::string*>(event->getUserData());
    
    int vc = PokktManager::getInstance()->getVideoVc();
    
    // enable buttons
    CCLOG("[POKKT-CPP] enabling buttons! Video Vc is %d", vc);
    
    _txtCacheMessage->setString("Video downloaded!");
    
    _btnPlayVideo->setTitleText("Play Video to Earn " + coinsToEarn);
    _btnPlayVideo->setVisible(true);
    _btnPlayVideoNonIncent->setVisible(true);
    //_btnPlayVideoWithIntent->setVisible(true);
}

void VideoView::handleDownloadFailed(cocos2d::Ref *pSender)
{
    CCLOG("[POKKT-CPP] Failed to download video!");
    _txtCacheMessage->setString("Failed to download video!");
}
