#ifndef __CocosPokktApp__PokktManager__
#define __CocosPokktApp__PokktManager__

#include <stdio.h>
#include <stdlib.h>
#include "cocos2d.h"
#include "PokktNativeExtension.h"

class PokktManager
{
    friend class PokktNativeExtension;
    
public:
    
    // events
    static const std::string VideoClosedEvent;
    static const std::string VideoDisplayedEvent;
    static const std::string VideoSkippedEvent;
    static const std::string VideoCompletedEvent;
    static const std::string VideoGratifiedEvent;
    static const std::string ShowIntent;
    static const std::string DownloadCompletedEvent;
    static const std::string DownloadFailedEvent;
    static const std::string CoinResponseEvent;
    static const std::string CoinResponseWithTransIdEvent;
    static const std::string CoinResponseFailedEvent;
    static const std::string CampaignAvailabilityEvent;
    static const std::string OfferwallClosedEvent;
    
    // pokkt params
    static std::string SecurityKey;
    static std::string ApplicationId;
    static std::string UserId;
    static std::string IntegrationType;
    static std::string AutoCacheVideo;
    
    static PokktManager* getInstance();
    
    // video caching mode
    void setAutoCaching(bool autoCaching);
    bool getAutoCaching();
    
    // misc
    void setParams(std::string params);
    void setDebug(bool value);
    void showLog(std::string message);
    void showToast(std::string message);
    
    // pokkt operations
    void getFreeCoins();
    void getFreeCoins(const std::string &fixedAssetValue);
    void getPendingCoins();
    void checkOfferwallCampaign();
    void getVideo();
    void getVideoNonIncent();
    void getVideoWithIntent();
    void startVideoCaching();
    void setSkipVideo(bool hasSkipped);
    void setDefaultSkipTime(std::string time);
    void setAge(std::string age);
    void setBirthday(std::string birthday);
    void setContentType(std::string contentType);
    void setEducationType(std::string educationType);
    void setEmploymentStatus(std::string empStatus);
    void setFacebookId(std::string fbId);
    void setLocation(std::string location);
    void setMaritalStatus(std::string maritalStatus);
    void setName(std::string name);
    void setNationality(std::string nationality);
    void setSex(std::string sex);
    void setThePubs(std::string pubs);
    void setThirdPartyUserId(std::string thirdPartyUserId);
    void setTwitterHandle(std::string twitterHandle);
    bool isVideoAvailable();
    int getVideoVc();
    
    static inline std::string getVersion() { return "1.0.0"; }
    
    ~PokktManager(){}
    
//protected:
    
    // handles messages coming from native pokkt runtime
    void handleNativeMessage(std::string operation, std::string param);
    
private:
    
    static PokktManager* _instance;
    PokktNativeExtension* _nativeExtension;
    bool _autoCaching;
    
    PokktManager();
    
    std::string getParamString();
};

#endif /* defined(__CocosPokktApp__PokktManager__) */
