#include "PokktManager.h"

// pokkt event definitions
const std::string PokktManager::VideoClosedEvent = "videoClosed";
const std::string PokktManager::VideoDisplayedEvent = "videoDisplayed";
const std::string PokktManager::VideoSkippedEvent = "videoSkipped";
const std::string PokktManager::VideoCompletedEvent = "videoCompleted";
const std::string PokktManager::VideoGratifiedEvent = "videoGratified";
const std::string PokktManager::ShowIntent = "showIntent";
const std::string PokktManager::DownloadCompletedEvent = "downloadCompleted";
const std::string PokktManager::DownloadFailedEvent = "downloadFailed";
const std::string PokktManager::CoinResponseEvent = "coinResponse";
const std::string PokktManager::CoinResponseWithTransIdEvent = "coinResponseWithTrId";
const std::string PokktManager::CoinResponseFailedEvent = "coinResponseFailed";
const std::string PokktManager::CampaignAvailabilityEvent = "campaignAvailability";
const std::string PokktManager::OfferwallClosedEvent = "offerwallClosedEvent";

std::string PokktManager::SecurityKey  = "";
std::string PokktManager::ApplicationId  = "";
std::string PokktManager::UserId  = "";
std::string PokktManager::IntegrationType  = "";
std::string PokktManager::AutoCacheVideo  = "";

// sigleton instance
PokktManager *PokktManager::_instance = 0;
PokktManager *PokktManager::getInstance()
{
    if (_instance == NULL)
        _instance = new PokktManager();
    return _instance;
}

PokktManager::PokktManager():
_nativeExtension(new PokktNativeExtension()),
_autoCaching(true)
{
    CCLOG("[POKKT-CPP] init...");
}

void PokktManager::handleNativeMessage(std::string operation, std::string param)
{
    CCLOG("[POKKT-CPP] message received! operation: %s, param: %s", operation.c_str(), param.c_str());
    auto dispatcher = cocos2d::Director::getInstance()->getEventDispatcher();
    dispatcher->dispatchCustomEvent(operation, &param);
}

void PokktManager::setAutoCaching(bool autoCaching)
{
    _autoCaching = autoCaching;
    setParams(_autoCaching ? "" : getParamString());
}

bool PokktManager::getAutoCaching()
{
    return _autoCaching;
}

std::string PokktManager::getParamString()
{
    std::string paramString =
        PokktManager::SecurityKey + "," +
        PokktManager::ApplicationId + "," +
        PokktManager::UserId + "," +
        PokktManager::IntegrationType + "," +
        PokktManager::AutoCacheVideo;
    
    return paramString;
}

void PokktManager::setParams(std::string params)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetParams, params);
}

void PokktManager::setDebug(bool value)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetDebug, value ? "true" : "false");
}

void PokktManager::showLog(std::string message)
{
    _nativeExtension->performOperation(PokktNativeExtension::ShowLog, message);
}

void PokktManager::showToast(std::string message)
{
    _nativeExtension->performOperation(PokktNativeExtension::ShowToast, message);
}

void PokktManager::getFreeCoins()
{
    this->getFreeCoins("");
}

void PokktManager::getFreeCoins(const std::string &fixedAssetValue)
{
    _nativeExtension->performOperation(PokktNativeExtension::GetFreeCoins, fixedAssetValue);
}

void PokktManager::getPendingCoins()
{
    _nativeExtension->performOperation(PokktNativeExtension::GetPendingCoins);
}

void PokktManager::checkOfferwallCampaign()
{
    _nativeExtension->performOperation(PokktNativeExtension::CheckOfferwallCampaign);
}

void PokktManager::getVideo()
{
    _nativeExtension->performOperation(PokktNativeExtension::GetVideo);
}

void PokktManager::getVideoNonIncent()
{
    _nativeExtension->performOperation(PokktNativeExtension::GetVideoNonIncent);
}

void PokktManager::getVideoWithIntent()
{
    _nativeExtension->performOperation(PokktNativeExtension::GetVideoWithIntent);
}

void PokktManager::startVideoCaching()
{
    _nativeExtension->performOperation(PokktNativeExtension::StartVideoCaching);
}

void PokktManager::setSkipVideo(bool hasSkipped)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetSkipVideo, (hasSkipped ? "true" : "false"));
}

void PokktManager::setDefaultSkipTime(std::string time)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetDefaultSkipTime, time);
}

void PokktManager::setAge(std::string age)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetAge, age);
}

void PokktManager::setBirthday(std::string birthdayvalue)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetBirthday, birthdayvalue);
}

void PokktManager::setContentType(std::string contentType)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetContentType, contentType);
}

void PokktManager::setEducationType(std::string educationType)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetEducationType, educationType);
}

void PokktManager::setEmploymentStatus(std::string empStatus)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetEmploymentStatus, empStatus);
}

void PokktManager::setFacebookId(std::string fbId)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetFacebookId, fbId);
}

void PokktManager::setLocation(std::string location)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetLocation, location);
}

void PokktManager::setMaritalStatus(std::string maritalStatus)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetMaritalStatus, maritalStatus);
}

void PokktManager::setName(std::string name)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetName, name);
}

void PokktManager::setNationality(std::string nationality)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetNationality, nationality);
}

void PokktManager::setSex(std::string sex)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetSex, sex);
}

void PokktManager::setThePubs(std::string pubs)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetThePubs, pubs);
}

void PokktManager::setThirdPartyUserId(std::string thirdPartyUserId)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetThirdPartyUserId, thirdPartyUserId);
}

void PokktManager::setTwitterHandle(std::string twitterHandle)
{
    _nativeExtension->performOperation(PokktNativeExtension::SetTwitterHandle, twitterHandle);
}

bool PokktManager::isVideoAvailable()
{
    return _nativeExtension->isVideoAvailable();
}

int PokktManager::getVideoVc()
{
    return _nativeExtension->getVideoVc();
}
