#include "PokktNativeExtension.h"
#include "PokktManager.h"

/**
 HANDLES NATIVE ANDROID MESSAGES
 */
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)

#include "platform/android/jni/JniHelper.h"


#ifdef __cplusplus
extern "C"
{
#endif
    
    void Java_com_pokkt_cocos2dx_PokktNativeExtension_NotifyCPP(JNIEnv* env, jobject thiz, jstring operation, jstring param)
    {
        std::string operationString = cocos2d::JniHelper::jstring2string(operation);
        std::string paramString = cocos2d::JniHelper::jstring2string(param);
        
        PokktManager::getInstance()->handleNativeMessage(operationString, paramString);
        
        env->DeleteLocalRef(operation);
        env->DeleteLocalRef(param);
    }
    
#ifdef __cplusplus
};
#endif


#endif /* if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) */

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)

void PokktNativeExtension::notifyAndroid(std::string operation, std::string param)
{
    CCLOG("[POKKT-CPP] notifying android of operation: %s, param: %s", operation.c_str(), param.c_str());
    
    cocos2d::JniMethodInfo methodInfo;
    if (!cocos2d::JniHelper::getStaticMethodInfo(methodInfo,
                                                 "com/pokkt/cocos2dx/PokktNativeExtension",
                                                 "NotifyAndroid",
                                                 "(Ljava/lang/String;Ljava/lang/String;)V"))
        return;
    
    jstring joperation = methodInfo.env->NewStringUTF(operation.c_str());
    jstring jparam = methodInfo.env->NewStringUTF(param.c_str());
    methodInfo.env->CallStaticVoidMethod(methodInfo.classID, methodInfo.methodID, joperation, jparam);
    
    methodInfo.env->DeleteLocalRef(methodInfo.classID);
    methodInfo.env->DeleteLocalRef(joperation);
    methodInfo.env->DeleteLocalRef(jparam);
}

bool PokktNativeExtension::isVideoAvailableOnAndroid()
{
    CCLOG("[POKKT-CPP] checking for video availability on android device...");
    
    cocos2d::JniMethodInfo methodInfo;
    if (!cocos2d::JniHelper::getStaticMethodInfo(methodInfo,
                                                 "com/pokkt/cocos2dx/PokktNativeExtension",
                                                 "isVideoAvailableOnAndroid",
                                                 "()Z"))
        return false;
    
    jboolean jresult = methodInfo.env->CallStaticBooleanMethod(methodInfo.classID, methodInfo.methodID);
    bool result = (bool)jresult;
    
    methodInfo.env->DeleteLocalRef(methodInfo.classID);
    
    CCLOG(result ? "[POKKT-CPP] video found on android deivce!" : "[POKKT-CPP] video not available on android deivce!");
    
    return result;
}

int PokktNativeExtension::getVideoVcOnAndroid()
{
    CCLOG("[POKKT-CPP] getting video-vc on android device...");
    
    cocos2d::JniMethodInfo methodInfo;
    if (!cocos2d::JniHelper::getStaticMethodInfo(methodInfo,
                                                 "com/pokkt/cocos2dx/PokktNativeExtension",
                                                 "getVideoVcOnAndroid",
                                                 "()I"))
        return 0;
    
    jint jresult = methodInfo.env->CallStaticIntMethod(methodInfo.classID, methodInfo.methodID);
    int result = (int)jresult;
    
    methodInfo.env->DeleteLocalRef(methodInfo.classID);
    
    CCLOG("[POKKT-CPP] video-vc on android deivce currently is %d!", result);
    
    return result;
}

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)

void PokktNativeExtension::notifyIOS(std::string operation, std::string param)
{
    CCLOG("[POKKT-CPP] [TODO] notifying iOS of operation: %s, param: %s", operation.c_str(), param.c_str());
    // TODO
}

bool PokktNativeExtension::isVideoAvailableOnIOS()
{
    // TODO
    CCLOG("[POKKT-CPP] [TODO] checking for video availability on IOS device...");
    //CCLOG("[POKKT-CPP] video found on IOS deivce!");
    //CCLOG("[POKKT-CPP] video not available on IOS deivce!");
    return false;
}

int PokktNativeExtension::getVideoVcOnIOS()
{
    // TODO
    CCLOG("[POKKT-CPP] [TODO] getting video-vc on IOS device...");
    return 0;
}

#endif

void PokktNativeExtension::notifyNative(std::string operation, std::string param)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    notifyAndroid(operation, param);
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    notifyIOS(operation, param);
    
#endif
}

bool PokktNativeExtension::isVideoAvailable()
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    return isVideoAvailableOnAndroid();
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    return isVideoAvailableOnIOS();
    
#endif
}

int PokktNativeExtension::getVideoVc()
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    return getVideoVcOnAndroid();
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    return getVideoVcOnIOS();
    
#endif
}

void PokktNativeExtension::performOperation(PokktOperations operation, std::string param)
{
    switch (operation)
    {
        case SetParams:                 notifyNative("setParams",               param);   break;
        case SetDebug:                  notifyNative("setDebug",                param);   break;
        case ShowLog:                   notifyNative("showLog",                 param);   break;
        case ShowToast:                 notifyNative("showToast",               param);   break;
            
        case GetFreeCoins:              notifyNative("getFreeCoins",            param);   break;
        case GetPendingCoins:           notifyNative("getPendingCoins",     	param);   break;
        case CheckOfferwallCampaign:    notifyNative("checkOfferwallCampaign",  param);   break;
        case GetVideo:                  notifyNative("getVideo",            	param);   break;
        case GetVideoNonIncent:    		notifyNative("getVideoNonIncent",   	param);   break;
        case GetVideoWithIntent:    	notifyNative("getVideoWithIntent",      param);   break;
            
        case StartVideoCaching:         notifyNative("startVideoCaching",       param);   break;
        case SetSkipVideo:          	notifyNative("setSkipVideo",        	param);   break;
        case SetDefaultSkipTime:    	notifyNative("setDefaultSkipTime",      param);   break;
        case SetAge:                    notifyNative("setAge",              	param);   break;
        case SetBirthday:               notifyNative("setBirthday",             param);   break;
        case SetContentType:            notifyNative("setContentType",          param);   break;
        case SetEducationType:          notifyNative("setEducationType",    	param);   break;
        case SetEmploymentStatus:       notifyNative("setEmploymentStatus",     param);   break;
        case SetFacebookId:             notifyNative("setFacebookId",           param);   break;
        case SetLocation:               notifyNative("setLocation",             param);   break;
        case SetMaritalStatus:          notifyNative("setMaritalStatus",        param);   break;
        case SetName:                   notifyNative("setName",                 param);   break;
        case SetNationality:            notifyNative("setNationality",          param);   break;
        case SetSex:                    notifyNative("setSex",              	param);   break;
        case SetThePubs:                notifyNative("setThePubs",              param);   break;
        case SetThirdPartyUserId:       notifyNative("setThirdPartyUserId",     param);   break;
        case SetTwitterHandle:          notifyNative("setTwitterHandle",        param);   break;
    }
}
